@@ -31,7 +31,7 @@ fi
 echo $$>${LOCKFILE}
 
 # The following variables can be overridden with a configuration file
-TSBRANCH=lp:ubiquity
+TSBRANCH=https://git.launchpad.net/ubiquity
 EXTRAPACKAGES=""
 ARTIFACTS=""
 AP_OPTS="-v"
@@ -86,6 +86,10 @@ elif [ -n "${DESKTOP_SESSION+1}" ]; then
         gnome) #ubuntu-gnome
             SESSION_LOG=/var/log/apt/term.log
             ARTIFACTS="$ARTIFACTS /var/log/apt"
+            gsettings set org.gnome.desktop.session idle-delay 0
+            ;;
+        ubuntu)
+            gsettings set org.gnome.desktop.session idle-delay 0
             ;;
     esac
 else
@@ -93,7 +97,7 @@ else
     exit 1
 fi
 
-PACKAGES="bzr ssh python3-autopilot libautopilot-gtk python3-xlib \
+PACKAGES="git ssh python3-autopilot libautopilot-gtk python3-xlib \
     recordmydesktop"
 
 export DEBIAN_FRONTEND=noninteractive
@@ -208,8 +212,6 @@ setup_tests() {
     echo "I: Installating additional packages"
     retry_cmd 3 30 sudo apt-get update
     retry_cmd 3 30 sudo apt-get install -yq $PACKAGES $EXTRAPACKAGES
-    echo "I: Purging ubiquity-slideshow"
-    sudo apt-get autoremove --purge -y $(dpkg -l "ubiquity-slideshow-*"|awk '/^ii/ {print $2}')||true
 
     if [ -n "$SHAREDVOL" ]; then
         echo "I: Mounting $SHAREDVOL on $TSEXPORT"
@@ -218,7 +220,7 @@ setup_tests() {
         sudo chmod 777 $TSEXPORT
     else
         echo "I: Branch $TSBRANCH"
-        bzr export $TSEXPORT $TSBRANCH
+        git clone --depth 1 $TSBRANCH $TSEXPORT
     fi
 
     if [ -e "$AP_TESTSUITES" ]; then
@@ -229,7 +231,6 @@ setup_tests() {
 Image Id:      $(cat /cdrom/.disk/info)
 Ubiquity:      $(dpkg-query -f '${Version}' -W ubiquity)
 Test branch:   ${TSBRANCH}
-Test revno:    $(bzr revno $TSBRANCH)
 EOF
     
     cat<<EOF
@@ -297,9 +298,9 @@ run_tests() {
     tail_logs /var/log/installer/debug
     for testfile in $(ls -d $spooldir/* 2>/dev/null); do
         testname=$(basename $testfile)
-        echo "I: Running autopilot run $testname $AP_OPTS -o $AP_RESULTS/$testname.xml"
+        echo "I: Running autopilot run $testname $AP_OPTS -f subunit -o $AP_RESULTS/$testname.subunit"
         aprc=0
-        timeout -s 9 -k 30 $TIMEOUT ./autopilot run $testname $AP_OPTS -f xml -o $AP_RESULTS/${testname}.xml||aprc=$?
+        timeout -s 9 -k 30 $TIMEOUT ./autopilot run $testname $AP_OPTS -f subunit -o $AP_RESULTS/${testname}.subunit||aprc=$?
         if [ $aprc -gt 0 ]; then
             echo "${testname}: FAIL" >> $AP_SUMMARY
         else
