@@ -19,6 +19,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
+import grp
 import gzip
 import io
 import itertools
@@ -1013,7 +1014,7 @@ class Install(install_misc.InstallBase):
         """ Join Active Directory domain and enable pam_mkhomedir """
         use_directory = self.db.get('ubiquity/login_use_directory')
         if use_directory != 'true':
-            install_misc.record_removed(['adcli', 'krb5-config', 'realmd', 'sssd'], recursive=True)
+            install_misc.record_removed(['adcli', 'realmd', 'sssd'], recursive=True)
             return
 
         from socket import gethostname
@@ -1075,24 +1076,6 @@ class Install(install_misc.InstallBase):
             return False
 
         syslog.syslog(' '.join(log_args))
-
-        # Configure krb5 client so access to AD with GPO enabled works out of
-        # the box after installation.
-
-        # Split hostname and domainname
-        krb5conf = os.path.join(self.target, 'etc', 'krb5.conf')
-        dn = ''
-        if '.' in directory_domain:
-            dn = '.'.join(directory_domain.split('.')[1:]).upper()
-        install_misc.set_debconf(self.target, 'krb5-config/default_realm', dn, self.db)
-        install_misc.set_debconf(self.target, 'krb5-config/add_servers_realm', dn, self.db)
-        install_misc.set_debconf(self.target, 'krb5-config/admin_server', directory_domain, self.db)
-        install_misc.set_debconf(self.target, 'krb5-config/kerberos_servers', directory_domain, self.db)
-
-        os.unlink(krb5conf)
-
-        install_misc.reconfigure(self.target, 'krb5-config')
-
         return True
 
     def copy_mok(self):
@@ -1774,7 +1757,8 @@ class Install(install_misc.InstallBase):
                 if not misc.execute('cp', '-a', log_file, target_log_file):
                     syslog.syslog(syslog.LOG_ERR,
                                   'Failed to copy installation log file')
-                os.chmod(target_log_file, stat.S_IRUSR | stat.S_IWUSR)
+                os.chmod(target_log_file, 0o640)
+                os.chown(target_log_file, -1, grp.getgrnam('adm').gr_gid)
         media_info = '/cdrom/.disk/info'
         if os.path.isfile(media_info):
             try:
