@@ -6,6 +6,8 @@ from test.support import run_unittest
 import unittest
 
 import dbus
+import gi
+gi.require_version('Gtk', '3.0')
 from gi.repository import Gtk, TimezoneMap
 import mock
 
@@ -56,7 +58,7 @@ class WidgetTests(unittest.TestCase):
     def test_timezone_map(self):
         tzmap = TimezoneMap.TimezoneMap()
         self.win.add(tzmap)
-        #tzmap.select_city('America/New_York')
+        # tzmap.select_city('America/New_York')
         self.win.show_all()
         self.win.connect('destroy', Gtk.main_quit)
         gtkwidgets.refresh()
@@ -258,5 +260,6 @@ class NetworkManagerTests(unittest.TestCase):
         tv.data_func(None, mock_cell, self.model, i, None)
         mock_cell.set_property.assert_called_with('text', 'Orange')
 
+
 if __name__ == '__main__':
     run_unittest(WidgetTests, NetworkStoreTests, NetworkManagerTests)
