@@ -208,8 +208,8 @@ class TestPage(TestPageBase):
         self.assertEqual(no_detected, head)
 
 
-@unittest.skipUnless(os.environ['DEB_HOST_ARCH'] in ('amd64', 'i386'),
-                     'GRUB-related tests are only relevant on x86')
+@unittest.skipUnless(os.environ['DEB_HOST_ARCH'] in ('amd64', 'arm64', 'i386'),
+                     'GRUB-related tests are only relevant on EFI')
 class TestPageGrub(TestPageBase):
     def test_maybe_update_dont_install(self):
         self.page.install_bootloader = False
@@ -568,14 +568,6 @@ class TestCalculateAutopartitioningOptions(unittest.TestCase):
         desc = self.page.extended_description(question)
         use_device = ubi_partman.PartitioningOption(title, desc)
 
-        question = 'ubiquity/partitioner/ubuntu_upgrade'
-        question_has_variables(question, ['CURDISTRO', 'VER'])
-        self.page.db.subst(question, 'CURDISTRO', operating_system)
-        self.page.db.subst(question, 'VER', self.release.version)
-        title = self.page.description(question)
-        desc = self.page.extended_description(question)
-        reuse = ubi_partman.PartitioningOption(title, desc)
-
         operating_systems, ubuntu_systems = \
             self.page.calculate_operating_systems(layout)
         options = self.page.calculate_autopartitioning_options(
@@ -590,9 +582,6 @@ class TestCalculateAutopartitioningOptions(unittest.TestCase):
         self.assertIn('manual', options)
         self.assertCountEqual(self.manual, options['manual'])
 
-        self.assertIn('reuse', options)
-        self.assertCountEqual(reuse, options['reuse'])
-
     # 'This computer currently has Ubuntu 12.04 on it.'
     def test_same_ubuntu_only(self):
         operating_system = 'Ubuntu 12.04'
@@ -783,8 +772,9 @@ class TestPageGtk(unittest.TestCase):
                      '/dev/vda1', '/dev/vda2', '/dev/vdb1'),
                     ('Virtio Block Device (108 GB)',
                      'Virtio Block Device (801 GB)')))
-    @unittest.skipUnless(os.environ['DEB_HOST_ARCH'] in ('amd64', 'i386'),
-                         'GRUB-related tests are only relevant on x86')
+    @unittest.skipUnless(os.environ['DEB_HOST_ARCH'] in (
+        'amd64', 'arm64', 'i386'),
+        'GRUB-related tests are only relevant on EFI')
     def test_boot_loader_installation_combobox(self):
         self.gtk.set_grub_options('/dev/vda', {
             '/dev/vda': True,
