@@ -25,7 +25,7 @@ from ubiquity.filteredcommand import FilteredCommand
 
 class Install(FilteredCommand):
     def prepare(self, unfiltered=False):
-        automatic_mode = 'UBIQUITY_AUTOMATIC' in os.environ
+        automatic_mode = self.is_automatic
 
         if os.access('/usr/share/grub-installer/grub-installer', os.X_OK):
             with_other_os = self.db.get('grub-installer/with_other_os')
@@ -65,6 +65,9 @@ class Install(FilteredCommand):
             return True
         elif question == 'apt-setup/security-updates-failed':
             fatal = False
+        elif (question == 'ubiquity/install/broken_active_directory' or
+              question == 'ubiquity/install/broken_luks_add_key'):
+            fatal = False
         else:
             fatal = True
         self.frontend.error_dialog(self.description(question),
