@@ -17,14 +17,15 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-from __future__ import print_function
-
+import glob
 import os
 import subprocess
 import sys
+import syslog
 
 from ubiquity import i18n, misc, osextras, plugin, upower
-from ubiquity.install_misc import archdetect, is_secure_boot
+from ubiquity.install_misc import (archdetect, is_secure_boot,
+                                   minimal_install_rlist_path)
 
 NAME = 'prepare'
 AFTER = 'wireless'
@@ -39,15 +40,24 @@ OEM = False
 
 class PreparePageBase(plugin.PluginUI):
     plugin_title = 'ubiquity/text/prepare_heading_label'
+    download_updates = True
+    download_updates_enabled = True
 
     def __init__(self, *args, **kwargs):
         plugin.PluginUI.__init__(self)
 
     def plugin_set_online_state(self, state):
+        # if we're disabling, remember what it was before so we can re-enable
+        # properly if network comes back
+        if self.prepare_network_connection.get_state() and not state:
+            self.download_updates = self.get_download_updates()
         self.prepare_network_connection.set_state(state)
         self.enable_download_updates(state)
+        self.download_updates_enabled = state
         if not state:
             self.set_download_updates(False)
+        else:
+            self.set_download_updates(self.download_updates)
 
     def set_sufficient_space(self, state, required, free):
         if not state:
@@ -58,6 +68,13 @@ class PreparePageBase(plugin.PluginUI):
         self.prepare_sufficient_space.set_state(state)
 
     def plugin_translate(self, lang):
+        self.rst_title_text = i18n.get_string('rst_header', lang)
+        return
+
+    def show_ubuntu_drivers_spinner(self):
+        return
+
+    def hide_ubuntu_drivers_spinner(self):
         return
 
 
@@ -65,9 +82,6 @@ class PageGtk(PreparePageBase):
     restricted_package_name = 'ubuntu-restricted-addons'
 
     def __init__(self, controller, *args, **kwargs):
-        if self.is_automatic:
-            self.page = None
-            return
         self.controller = controller
         from ubiquity.gtkwidgets import Builder
         builder = Builder()
@@ -104,11 +118,21 @@ class PageGtk(PreparePageBase):
         self.secureboot_box.set_sensitive(False)
         self.password_grid.set_sensitive(False)
 
+        self.minimal_install_vbox.set_visible(
+            os.path.exists(minimal_install_rlist_path))
+
         self.prepare_page = builder.get_object('stepPrepare')
         self.insufficient_space_page = builder.get_object('stepNoSpace')
+        self.rst_page = builder.get_object('stepRST')
+        self.rst_label = builder.get_object('label_using_rst')
+        self.rst_label.connect('activate-link', self.on_link_clicked)
         self.current_page = self.prepare_page
         self.plugin_widgets = self.prepare_page
-        self.plugin_optional_widgets = [self.insufficient_space_page]
+        self.plugin_optional_widgets = [self.insufficient_space_page,
+                                        self.rst_page]
+
+    def on_link_clicked(self, widget, uri):
+        misc.launch_uri(uri)
 
     def plugin_get_current_page(self):
         return self.current_page
@@ -121,6 +145,36 @@ class PageGtk(PreparePageBase):
 
         self.controller.go_to_page(self.current_page)
 
+    def show_rst_page(self):
+        for page in self.controller._wizard.pages:
+            if page.module.NAME == NAME:
+                page.title = 'ubiquity/text/rst_header'
+                break
+        self.current_page = self.rst_page
+        self.controller.go_to_page(self.current_page)
+        return True
+
+    def show_ubuntu_drivers_spinner(self):
+        frontend = self.controller._wizard
+        frontend.status_spinner.show()
+        frontend.status_spinner.start()
+        frontend.status_label.show()
+
+    def hide_ubuntu_drivers_spinner(self):
+        frontend = self.controller._wizard
+        frontend.status_spinner.hide()
+        frontend.status_spinner.stop()
+        frontend.status_label.hide()
+
+    def plugin_on_next_clicked(self):
+        if self.current_page != self.rst_page:
+            if self.get_use_nonfree():
+                self.show_ubuntu_drivers_spinner()
+            return
+
+        self.controller._wizard.do_reboot()
+        return True
+
     def set_using_secureboot(self, secureboot):
         self.using_secureboot = secureboot
         self.secureboot_box.set_visible(secureboot)
@@ -143,6 +197,12 @@ class PageGtk(PreparePageBase):
     def get_download_updates(self):
         return self.prepare_download_updates.get_active()
 
+    def set_minimal_install(self, val):
+        self.prepare_minimal_install.set_active(val)
+
+    def get_minimal_install(self):
+        return self.prepare_minimal_install.get_active()
+
     def set_allow_nonfree(self, allow):
         if not allow:
             self.prepare_nonfree_software.set_active(False)
@@ -163,6 +223,11 @@ class PageGtk(PreparePageBase):
 
     def plugin_translate(self, lang):
         PreparePageBase.plugin_translate(self, lang)
+
+        frontend = self.controller._wizard
+        frontend.status_label.set_text(self.controller.get_string(
+            'ubiquity/text/preparing_ud_label', lang))
+
         release = misc.get_release()
 
         from gi.repository import Gtk
@@ -247,14 +312,16 @@ class PageKde(PreparePageBase):
 
     def __init__(self, controller, *args, **kwargs):
         from ubiquity.qtwidgets import StateBox
-        if self.is_automatic:
-            self.page = None
-            return
         self.controller = controller
         try:
             from PyQt5 import uic
-            from PyQt5 import QtGui
+            from PyQt5 import QtGui, QtWidgets
+            # No worries, this has nothing to do with NM, we just want the
+            # generic progress indicator from there
+            from ubiquity.frontend.kde_components.nmwidgets import ProgressIndicator
             self.page = uic.loadUi('/usr/share/ubiquity/qt/stepPrepare.ui')
+            self.prepare_minimal_install = self.page.prepare_minimal_install
+            self.qt_label_minimal_install = self.page.qt_label_minimal_install
             self.prepare_download_updates = self.page.prepare_download_updates
             self.prepare_nonfree_software = self.page.prepare_nonfree_software
             self.prepare_foss_disclaimer = self.page.prepare_foss_disclaimer
@@ -267,6 +334,10 @@ class PageKde(PreparePageBase):
             self.badPassword = self.page.badPassword
             self.badPassword.setPixmap(QtGui.QPixmap(
                 "/usr/share/icons/oxygen/16x16/status/dialog-warning.png"))
+            self.progress_indicator = ProgressIndicator()
+            self.progress_indicator.hide()
+            layout = QtWidgets.QHBoxLayout(self.page.progress_container)
+            layout.addWidget(self.progress_indicator)
             # TODO we should set these up and tear them down while on this
             # page.
             try:
@@ -278,6 +349,9 @@ class PageKde(PreparePageBase):
             except Exception as e:
                 # TODO use an inconsistent state?
                 print('unable to set up power source watch:', e)
+            if not os.path.exists(minimal_install_rlist_path):
+                self.qt_label_minimal_install.hide()
+                self.prepare_minimal_install.hide()
             try:
                 self.prepare_network_connection = StateBox(self.page)
             except Exception as e:
@@ -289,6 +363,21 @@ class PageKde(PreparePageBase):
         self.set_using_secureboot(False)
         self.plugin_widgets = self.page
 
+    def show_rst_page(self):
+        return False
+
+    def show_ubuntu_drivers_spinner(self):
+        self.progress_indicator.show()
+        self.progress_indicator.setSpinnerVisible(True)
+
+    def hide_ubuntu_drivers_spinner(self):
+        self.progress_indicator.setSpinnerVisible(False)
+        self.progress_indicator.hide()
+
+    def plugin_on_next_clicked(self):
+        if self.get_use_nonfree():
+            self.show_ubuntu_drivers_spinner()
+
     def show_insufficient_space_page(self, required, free):
         from PyQt5 import QtWidgets
         QtWidgets.QMessageBox.critical(self.page,
@@ -337,6 +426,14 @@ class PageKde(PreparePageBase):
         from PyQt5.QtCore import Qt
         return self.prepare_download_updates.checkState() == Qt.Checked
 
+    def set_minimal_install(self, val):
+        self.prepare_minimal_install.setChecked(val)
+
+    def get_minimal_install(self):
+        if self.prepare_minimal_install.isChecked():
+            return True
+        return False
+
     def set_allow_nonfree(self, allow):
         if not allow:
             self.prepare_nonfree_software.setChecked(False)
@@ -356,6 +453,9 @@ class PageKde(PreparePageBase):
 
     def plugin_translate(self, lang):
         PreparePageBase.plugin_translate(self, lang)
+        # Translate the progress label
+        self.progress_indicator.setText(self.controller.get_string(
+            'ubiquity/text/preparing_ud_label', lang))
         # gtk does the ${RELEASE} replace for the title in gtk_ui but we do
         # it per plugin because our title widget is per plugin
         release = misc.get_release()
@@ -384,10 +484,40 @@ class Page(plugin.Plugin):
             if is_secure_boot():
                 self.ui.set_using_secureboot(True)
 
-        download_updates = self.db.get('ubiquity/download_updates') == 'true'
-        self.ui.set_download_updates(download_updates)
+        self.ui.download_updates = self.db.get('ubiquity/download_updates') == 'true'
+        if self.ui.download_updates_enabled:
+            self.ui.set_download_updates(self.ui.download_updates)
+        minimal_install = self.db.get('ubiquity/minimal_install') == 'true'
+        self.ui.set_minimal_install(minimal_install)
         self.apply_debconf_branding()
-        self.setup_sufficient_space()
+
+        # wait for it to finish
+        if self.frontend.ubuntu_drivers:
+            self.frontend.ubuntu_drivers.communicate()
+            self.frontend.ubuntu_drivers = None
+
+        # output whether there are OEM packages for this system
+        try:
+            with open('/run/ubuntu-drivers-oem.autoinstall', 'r') as f:
+                syslog.syslog(F'ubuntu-drivers list-oem finished with: "{" ".join(f.read().splitlines())}"')
+        except FileNotFoundError:
+            syslog.syslog("ubuntu-drivers list-oem finished with no available packages. Maybe we need to apt update? "
+                          "Doing that and trying again.")
+            # We only do this when we really have to since it could be slow: apt update & re-run of ubuntu-drivers
+            self.frontend.save_oem_metapackages_list(wait_finished=True)
+            try:
+                with open('/run/ubuntu-drivers-oem.autoinstall', 'r') as f:
+                    syslog.syslog(F'ubuntu-drivers list-oem finished with: "{" ".join(f.read().splitlines())}"')
+            except FileNotFoundError:
+                syslog.syslog("No, we didn't find any OEM packages again.")
+
+        if self.should_show_rst_page():
+            if not self.ui.show_rst_page():
+                self.setup_sufficient_space()
+            else:
+                self.ui.plugin_is_restart = True
+        else:
+            self.setup_sufficient_space()
         command = ['/usr/share/ubiquity/simple-plugins', 'prepare']
         questions = ['ubiquity/use_nonfree']
         return command, questions
@@ -398,6 +528,14 @@ class Page(plugin.Plugin):
                          'ubiquity/text/free_space']:
             self.db.subst(template, 'RELEASE', release.name)
 
+    def should_show_rst_page(self):
+        search = '/sys/module/ahci/drivers/pci:ahci/*/remapped_nvme'
+        for remapped_nvme in glob.glob(search):
+            with open(remapped_nvme, 'r') as f:
+                if int(f.read()) > 0:
+                    return True
+        return os.environ.get('SHOW_RST_UI', '0') == '1'
+
     def setup_sufficient_space(self):
         # TODO move into prepare.
         size = misc.install_size()
@@ -426,10 +564,12 @@ class Page(plugin.Plugin):
 
     def ok_handler(self):
         download_updates = self.ui.get_download_updates()
+        minimal_install = self.ui.get_minimal_install()
         use_nonfree = self.ui.get_use_nonfree()
         secureboot_key = self.ui.get_secureboot_key()
         self.preseed_bool('ubiquity/use_nonfree', use_nonfree)
         self.preseed_bool('ubiquity/download_updates', download_updates)
+        self.preseed_bool('ubiquity/minimal_install', minimal_install)
         if self.ui.using_secureboot and secureboot_key:
             self.preseed('ubiquity/secureboot_key', secureboot_key, seen=True)
         if use_nonfree:
@@ -442,3 +582,7 @@ class Page(plugin.Plugin):
                         'ubiquity/nonfree_package',
                         self.ui.restricted_package_name)
         plugin.Plugin.ok_handler(self)
+
+    def cleanup(self):
+        self.ui.hide_ubuntu_drivers_spinner()
+        plugin.Plugin.cleanup(self)
